/*
 * Jmunit.java
 *  
 * Copyright 2008 C.A. Meijer
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jmunit.anttask.cldc10;

import java.util.Vector;

import jmunit.framework.cldc10.AssertionFailedException;
import jmunit.framework.cldc10.TestListener;
import jmunit.framework.cldc10.TestResult;

import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Reference;

/**
 * The Jmunit task runs JMUnit tests as an Ant task.
 * 
 * @author C.A. Meijer
 * @since JMUnit 1.1
 */
public class Jmunit extends Task implements TestListener {

	// Should the task print summaries of tests?
	private boolean printsummary = true;

	// The formatter to use.
	private Formatter formatter;

	// Should the build fail on an error?
	private boolean haltonerror;

	// Should the build fail on a failure?
	private boolean haltonfailure;

	// Was an error reported in the current test?
	private boolean error;

	// Was a failure reported in the current test?
	private boolean failure;

	// A property that must be set if a test fails.
	private String failureproperty;

	// A property that must be set if a test reports an error.
	private String errorproperty;

	// The tests that must be run.
	private Vector tests = new Vector();

	// The classpath where tests can be found.
	private Path classpath;

	/**
	 * Add a test to the task.
	 * 
	 * @param test
	 *            The test to run.
	 */
	public void addTest(Test test) {
		this.tests.add(test);
	}

	/**
	 * Sets whether a test summary must be printed after each test.
	 * 
	 * @param onOff
	 *            Must be "on" or "off".
	 */
	public void setPrintsummary(String onOff) {
		this.printsummary = "on".equals(onOff.toLowerCase());
	}

	/**
	 * A formatter to use for reporting tests.
	 * 
	 * @param formatter
	 *            The formatter.
	 */
	public void addFormatter(Formatter formatter) {
		this.formatter = formatter;
	}

	/**
	 * Sets whether the build must be stopped after an error.
	 * 
	 * @param haltonerror
	 *            Must be "true" or "false".
	 */
	public void setHaltonerror(String haltonerror) {
		this.haltonerror = haltonerror.equals("true");
	}

	/**
	 * Sets whether the build must be stopped after a failure.
	 * 
	 * @param haltonfailure
	 *            Must be "true" or "false".
	 */
	public void setHaltonfailure(String haltonfailure) {
		this.haltonfailure = haltonfailure.equals("true");
	}

	/**
	 * Sets a property that must be set if a test fails.
	 * 
	 * @param failureproperty
	 *            The name of the property.
	 */
	public void setFailureproperty(String failureproperty) {
		this.failureproperty = failureproperty;
	}

	/**
	 * Sets a property that must be set if a test reports an error.
	 * 
	 * @param errorproperty
	 *            The name of the property.
	 */
	public void setErrorproperty(String errorproperty) {
		this.errorproperty = errorproperty;
	}

	/**
	 * Sets the class path where the tests are located.
	 * 
	 * @param classpath
	 */
	public void setClasspath(Path classpath) {
		this.classpath = classpath;
	}

	/**
	 * Sets the class path where the tests are located.
	 * 
	 * @param classpath
	 */
	public void addClasspath(Path classpath) {
		setClasspath(classpath);
	}

	/**
	 * Sets the class path where the tests are located.
	 * 
	 * @param refid
	 *            A reference to the class path.
	 */
	public void setRefid(Reference refid) {
		Path classpath = new Path(getProject());
		classpath.setRefid(refid);
		setClasspath(classpath);
	}

	/**
	 * Runs the nested JMUnit tests.
	 */
	public void execute() {
		// Ensure that we can load the tests from the class path.
		AntClassLoader cl = new AntClassLoader(
				this.getClass().getClassLoader(), true);
		if (this.classpath != null) {
			cl.setClassPath(this.classpath);
		}

		// Run all the tests.
		for (int i = 0; i < this.tests.size(); i++) {
			Test testElement = (Test) this.tests.elementAt(i);
			String testClass = testElement.getName();
			try {
				jmunit.framework.cldc10.Test test = (jmunit.framework.cldc10.Test) cl
						.loadClass(testClass).newInstance();
				TestResult result = new TestResult();
				result.addListener(this);
				if (this.printsummary) {
					result.addListener(new PrintSummaryListener(test));
				}
				if (this.formatter != null) {
					result.addListener(new XmlReportListener(test, testElement
							.getTodir()));
				}
				test.run(result);

				if (this.haltonfailure && this.failure) {
					throw new BuildException("Test reported a failure.");
				}

				if (this.haltonerror && this.error) {
					throw new BuildException("Test reported an error.");
				}
			} catch (BuildException e) {
				throw e;
			} catch (Exception e) {
				throw new BuildException(e);
			}
		}
	}

	/**
	 * An error occurred.
	 * 
	 * @param testClass
	 *            The test that reported the error.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 * @param t
	 *            The error.
	 */
	public void addError(Class testClass, String testName, Throwable t) {
		this.error = true;
		if (this.errorproperty != null) {
			Project proj = getProject();
			proj.setProperty(this.errorproperty, "true");
		}

		// Errors are also regarded as failures.
		addFailure(testClass, testName, new AssertionFailedException());
	}

	/**
	 * A failure occurred.
	 * 
	 * @param testClass
	 *            The test that reported the failure.
	 * 
	 * @param testName
	 *            The name of the test case.
	 * 
	 * @param t
	 *            The failure.
	 */
	public void addFailure(Class testClass, String testName,
			AssertionFailedException t) {
		this.failure = true;
		if (this.failureproperty != null) {
			Project proj = getProject();
			proj.setProperty(this.failureproperty, "true");
		}
	}

	public void clear() {
	}

	public void endTest(Class arg0, String arg1) {
	}

	public void startTest(Class arg0, String arg1) {
	}

}
